<?php

namespace Teknisa\TCO\Util;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class TcoUtil extends \Teknisa\GER\Controllers\TeknisaCrud {
    
    protected $crudService;
    protected $environment;
    protected $entityManager;
    protected $dataSourceManager;
    protected $newCode;

    public function __construct($crudService, $parameterBag, $environment, $entityManager, $dataSourceManager, $newCode) {
        $this->crudService                          = $crudService;
        $this->environment                          = $environment;
        $this->entityManager                        = $entityManager;
        $this->dataSourceManager                    = $dataSourceManager;
        $this->newCode                              = $newCode;
    }
    
    public function getMaxLevelPrato(DTO\Request $request, DTO\Response $response){
        $dataSet = $request->getDataSet();
        $sql = "SELECT MAX_NIVEL(CDPICTPRAT) AS MAXLEVELPRATO FROM PARAMGERAL";
	    $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute();
        $result = $query->fetchAll();
        $response->addDataSet(new DataSet("getMaxLevelPrato", $result[0]));
    }
    
    public function getMaxLevelProduto(DTO\Request $request, DTO\Response $response){
        $dataSet = $request->getDataSet();
        $sql = "SELECT MAX_NIVEL(CDPICTPROD) AS MAXLEVELPROD FROM PARAMGERAL";
	    $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute();
        $result = $query->fetchAll();
        $response->addDataSet(new DataSet("getMaxLevelProduto", $result[0]));
    }
    
    public function getMaskProduto(DTO\Request $request, DTO\Response $response){
        $dataSet = $request->getDataSet();
        $sql = "SELECT CDPICTPROD FROM PARAMGERAL";
	    $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute();
        $result = $query->fetchAll();
        $response->addDataSet(new DataSet("getMaskProduto", $result[0]));
    }
    
    public function getMaskPrato(DTO\Request $request, DTO\Response $response){
        $dataSet = $request->getDataSet();
        $sql = "SELECT CDPICTPRAT FROM PARAMGERAL";
	    $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute();
        $result = $query->fetchAll();
        $response->addDataSet(new DataSet("getMaskPrato", $result[0]));
    }
    
    public function verifyCompCardTrabIsNew($row){
        $sql = "SELECT * FROM COMPCARDTRAB WHERE NRCOMPCARDTRAB = :NRCOMPCARDTRAB";
        $params = array(
            "NRCOMPCARDTRAB" => $row["NRCOMPCARDTRAB"]
        );
	    $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll();
        return empty($result);
    }
    
    /*Criado para manter a compatibilidade entre PHP5 e PHP7*/
    public function oldSizeof($value){
        if(!isset($value))
            return 0;
        else if (is_array($value))
            return sizeof($value);
        else
            return 1;
    }
    /*
        Função para casos onde o dataSource é buscado do backend, e o mesmo pode ser filtrado pelo floatingControl
        Consultar Wiki para exemplos
    */
    public function prepareQueryLikeAll($conditions, $assocFieldToTable, $valuesWithDistinctLabel = null, $propertysConcat = null){
        $queryLikeAll = '';
        if(!$conditions) return $queryLikeAll;
        $likeI = "";
        $condition = array();
        $columnsLikeI = array();
        $likeAll = false;
        foreach($conditions as $cond){
            if ($cond['operator'] === 'MAPPED_LIKE_ALL' && !$likeAll){
                $likeAll = true;
                $condition = $cond;
            } else if($cond['operator'] === 'LIKE_I' && array_search($cond['columnName'], $columnsLikeI) === false){
                $attr = isset($assocFieldToTable[$cond['columnName']]) ? $assocFieldToTable[$cond['columnName']] : null;
                if($attr){
                    $hadTableName = $attr['tableName'] !== "";
                    $tableName = $hadTableName ? $attr['tableName'].'.'.$cond['columnName'] : $cond['columnName']; 
                    if(isset($attr['format'])){
                        $item = $attr['format'];
                        $item = str_replace('tableName', $attr['tableName'], $item);
                        $item = str_replace('NAME_COLUMN', $cond['columnName'], $item);
                        $tableName = $item;
                    }
                    $likeI .= 'AND (UPPER(' . $tableName . ') LIKE UPPER(\''. $cond['value'] . '\')) ';
                    array_push($columnsLikeI, $cond['columnName']);
                }
            }
        }
        $queryLikeAll .= $likeI;
        if(empty($condition)) return $queryLikeAll;
        
        $columnName = array();
        $columnFilter = str_replace("&","|",str_replace("#", "|", $condition['columnName']));
        if($valuesWithDistinctLabel && !empty($valuesWithDistinctLabel)){
            foreach($valuesWithDistinctLabel as $currentColumn => $newColumn){
                $columnFilter = str_replace($currentColumn, $newColumn, $columnFilter);
            }
        }
        preg_match("/[A-Z|]+/", $columnFilter, $columnName);
        if(empty($columnName) || strlen($columnName[0]) == 0) return $queryLikeAll;
        
        $columnName = $columnName[0];
        $columns = explode("|", $columnName);
        $columnsToLike = array();
        foreach($columns as $column){
            $item = '';
            $attr = array();
            $ignore = false;
            $concat = false;
            if($propertysConcat != null && isset($propertysConcat[$column])){
                if($propertysConcat[$column] !== false){
                    $attr = $assocFieldToTable[$propertysConcat[$column]];
                    $concat = true;
                } else {
                    $ignore = true;
                } 
            } else {
                 $attr = $assocFieldToTable[$column];
            }
            if(!$ignore){
                if(isset($attr['tableName'])){
                    $item = $attr['tableName'].'.';
                    if(isset($attr['format'])){
                        $item = $attr['format'];
                        $item = str_replace('tableName', $attr['tableName'], $item);
                        $item = str_replace('NAME_COLUMN', $column, $item);
                    }
                    if($concat && isset($attr['concat'])){
                        $columnsTable = explode('_',$propertysConcat[$column]);
                        $item .= $columnsTable[0].'||\''.$attr['concat'].'\'||'.$attr['tableName'].'.'.$columnsTable[1]; 
                    } else if(!isset($attr['format'])){
                        $item .= $column; 
                    }
                }
                array_push($columnsToLike, $item);
            }
        }
        $columnsToLike = implode('||\'_\'||', $columnsToLike);
        $queryLikeAll .= 'AND '. 'UPPER('.$columnsToLike.')' . ' LIKE ' . 'UPPER(\''.$condition['value'][0]['_ALL'].'\')';
        return $queryLikeAll;
    }
    
    public static function getSmtpSettingsFromAddress($url){
        $match = self::splitByCommonDelimiters($url);
        
        if (empty($match)) {
            throw new \Exception('INVALID_SMTP_HOST');
        }
        
        if (count($match) > 1) {
            $params['port'] = $match[1];
        }
        
        $params['host'] = $match[0];
        
        return $params;
    }
    
    private static function splitByCommonDelimiters($string) {
        $pattern = '/[:;|]+/';
        $cleanedString = self::removeAllWhitespaces($string);
        $splittedString = preg_split($pattern, $cleanedString);
        return $splittedString;
    }
    
    private static function removeAllWhitespaces($string) {
        $pattern = '/\s+/';
        $cleanedString = preg_replace($pattern, '', $string);
        return $cleanedString;
    }
    
    public static function getAspUrlParameter() {
        return InstanceManager::getInstance()->getParameter('aspUrl');
    }

    public static function getDelphiPasswordEncrypted($password, $isEncrypt = true, $offset = '2')
    {
        $url =  self::getAspUrlParameter() . '/comwebservice/service.asmx?wsdl';
        $options = array(
            'encoding'    => 'ISO-8859-1',
            'trace'       => true,
            'exceptions'  => true,
            'compression' => SOAP_COMPRESSION_ACCEPT | SOAP_COMPRESSION_GZIP,
        );
    
        $args = array(
            new \SoapVar($password, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
            new \SoapVar($offset, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
        );
    
        $cryptType = $isEncrypt ? 'GetCripto' : 'GetUncripto';
        
        $params = array(
            'rdm'    => 'Bib.rdmDBGeral',
            'method' => $cryptType,
            'args'   => $args,
        );
    
        try {
            $client = new \SoapClient($url, $options);
            $result = $client->executeCOMMethod($params);
            $encryption = $result->executeCOMMethodResult;
        } catch (\SoapFault $e) {
            throw new \Exception($e->getMessage());
        }
    
        return $encryption;
    }
    
    public function novoCodigo($contador, $quantidade=1){
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'NOVO_CODIGO_NT');
        $procedure->addParam(new Param('P_CONTADOR',   Param::PARAM_INPUT,  $contador,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_SEQUENCIAL', Param::PARAM_OUTPUT,  NULL,      Param::PARAM_TYPE_STR, 20));
        $procedure->addParam(new Param('P_QTDE',       Param::PARAM_INPUT,  $quantidade, Param::PARAM_TYPE_STR));
        $resultado = $procedure->execute();
        return $resultado['P_SEQUENCIAL'];
    }
    
}